@@ -8,20 +8,24 @@ from copy import copy
 from functools import partial
 from pathlib import Path
 from types import MappingProxyType
-from typing import Any, Literal, Union
+from typing import Any, Literal
 
+import dask
+import datashader as ds
 import matplotlib
 import matplotlib.patches as mpatches
 import matplotlib.path as mpath
 import matplotlib.pyplot as plt
+import matplotlib.transforms as mtransforms
 import numpy as np
+import numpy.ma as ma
 import pandas as pd
 import shapely
 import spatialdata as sd
 import xarray as xr
 from anndata import AnnData
 from cycler import Cycler, cycler
-from datatree import DataTree
+from datashader.core import Canvas
 from geopandas import GeoDataFrame
 from matplotlib import colors, patheffects, rcParams
 from matplotlib.axes import Axes
@@ -32,7 +36,6 @@ from matplotlib.colors import (
     LinearSegmentedColormap,
     ListedColormap,
     Normalize,
-    TwoSlopeNorm,
     to_rgba,
 )
 from matplotlib.figure import Figure
@@ -51,12 +54,13 @@ from skimage.color import label2rgb
 from skimage.morphology import erosion, square
 from skimage.segmentation import find_boundaries
 from skimage.util import map_array
-from spatialdata import SpatialData, get_element_annotators, get_values, rasterize
+from spatialdata import SpatialData, get_element_annotators, get_extent, get_values, rasterize
 from spatialdata._core.query.relational_query import _locate_value, _ValueOrigin
 from spatialdata._types import ArrayLike
 from spatialdata.models import Image2DModel, Labels2DModel, PointsModel, SpatialElement, get_model
 from spatialdata.transformations.operations import get_transformation
-from xarray import DataArray
+from spatialdata.transformations.transformations import Scale
+from xarray import DataArray, DataTree
 
 from spatialdata_plot._logging import logger
 from spatialdata_plot.pl.render_params import (
@@ -77,7 +81,7 @@ to_hex = partial(colors.to_hex, keep_alpha=True)
 # replace with
 # from spatialdata._types import ColorLike
 # once https://github.com/scverse/spatialdata/pull/689/ is in a release
-ColorLike = Union[tuple[float, ...], str]
+ColorLike = tuple[float, ...] | str
 
 
 def _verify_plotting_tree(sdata: SpatialData) -> SpatialData:
@@ -339,7 +343,7 @@ def _get_collection_shape(
                 c = cmap(c)
             else:
                 try:
-                    norm = colors.Normalize(vmin=min(c), vmax=max(c))
+                    norm = colors.Normalize(vmin=min(c), vmax=max(c)) if norm is None else norm
                 except ValueError as e:
                     raise ValueError(
                         "Could not convert values in the `color` column to float, if `color` column represents"
@@ -353,7 +357,7 @@ def _get_collection_shape(
             c = cmap(c)
         else:
             try:
-                norm = colors.Normalize(vmin=min(c), vmax=max(c))
+                norm = colors.Normalize(vmin=min(c), vmax=max(c)) if norm is None else norm
             except ValueError as e:
                 raise ValueError(
                     "Could not convert values in the `color` column to float, if `color` column represents"
@@ -491,11 +495,8 @@ def _prepare_cmap_norm(
     cmap: Colormap | str | None = None,
     norm: Normalize | None = None,
     na_color: ColorLike | None = None,
-    vmin: float | None = None,
-    vmax: float | None = None,
-    vcenter: float | None = None,
-    **kwargs: Any,
 ) -> CmapParams:
+    # TODO: check refactoring norm out here as it gets overwritten later
     cmap_is_default = cmap is None
     if cmap is None:
         cmap = rcParams["image.cmap"]
@@ -505,13 +506,7 @@ def _prepare_cmap_norm(
     cmap = copy(cmap)
 
     if norm is None:
-        norm = Normalize(vmin=vmin, vmax=vmax, clip=True)
-    elif isinstance(norm, Normalize) or not norm:
-        pass  # TODO
-    elif vcenter is None:
-        norm = Normalize(vmin=vmin, vmax=vmax, clip=True)
-    else:
-        norm = TwoSlopeNorm(vmin=vmin, vmax=vmax, vcenter=vcenter)
+        norm = Normalize(vmin=None, vmax=None, clip=False)
 
     na_color, na_color_modified_by_user = _sanitise_na_color(na_color)
     cmap.set_bad(na_color)
@@ -531,7 +526,7 @@ def _set_outline(
     outline_color: str | list[float] = "#0000000ff",  # black, white
     **kwargs: Any,
 ) -> OutlineParams:
-    if not isinstance(outline_width, (int, float)):
+    if not isinstance(outline_width, int | float):
         raise TypeError(f"Invalid type of `outline_width`: {type(outline_width)}, expected `int` or `float`.")
     if outline_width == 0.0:
         outline = False
@@ -873,9 +868,9 @@ def _generate_base_categorial_color_mapping(
         na_color = to_hex(to_rgba(na_color)[:3])
 
         if na_color and len(categories) > len(colors):
-            return dict(zip(categories, colors + [na_color]))
+            return dict(zip(categories, colors + [na_color], strict=True))
 
-        return dict(zip(categories, colors))
+        return dict(zip(categories, colors, strict=True))
 
     return _get_default_categorial_color_mapping(color_source_vector)
 
@@ -892,7 +887,7 @@ def _modify_categorical_color_mapping(
         # subset base mapping to only those specified in groups
         modified_mapping = {key: mapping[key] for key in mapping if key in groups or key == "NaN"}
     elif len(palette) == len(groups) and isinstance(groups, list) and isinstance(palette, list):
-        modified_mapping = dict(zip(groups, palette))
+        modified_mapping = dict(zip(groups, palette, strict=True))
     else:
         raise ValueError(f"Expected palette to be of length `{len(groups)}`, found `{len(palette)}`.")
 
@@ -913,7 +908,10 @@ def _get_default_categorial_color_mapping(
         palette = ["grey" for _ in range(len_cat)]
         logger.info("input has more than 103 categories. Uniform 'grey' color will be used for all categories.")
 
-    return {cat: to_hex(to_rgba(col)[:3]) for cat, col in zip(color_source_vector.categories, palette[:len_cat])}
+    return {
+        cat: to_hex(to_rgba(col)[:3])
+        for cat, col in zip(color_source_vector.categories, palette[:len_cat], strict=True)
+    }
 
 
 def _get_categorical_color_mapping(
@@ -1347,7 +1345,7 @@ def _multiscale_to_spatial_image(
             optimal_index_x -= 1
 
         # pick the scale with higher resolution (worst case: downscaled afterwards)
-        optimal_scale = scales[min(optimal_index_x, optimal_index_y)]
+        optimal_scale = scales[min(int(optimal_index_x), int(optimal_index_y))]
 
     # NOTE: problematic if there are cases with > 1 data variable
     data_var_keys = list(multiscale_image[optimal_scale].data_vars)
@@ -1417,12 +1415,12 @@ def _validate_show_parameters(
     return_ax: bool,
     save: str | Path | None,
 ) -> None:
-    if coordinate_systems is not None and not isinstance(coordinate_systems, (list, str)):
+    if coordinate_systems is not None and not isinstance(coordinate_systems, list | str):
         raise TypeError("Parameter 'coordinate_systems' must be a string or a list of strings.")
 
     font_weights = ["light", "normal", "medium", "semibold", "bold", "heavy", "black"]
     if legend_fontweight is not None and (
-        not isinstance(legend_fontweight, (int, str))
+        not isinstance(legend_fontweight, int | str)
         or (isinstance(legend_fontweight, str) and legend_fontweight not in font_weights)
     ):
         readable_font_weights = ", ".join(font_weights[:-1]) + ", or " + font_weights[-1]
@@ -1434,7 +1432,7 @@ def _validate_show_parameters(
     font_sizes = ["xx-small", "x-small", "small", "medium", "large", "x-large", "xx-large"]
 
     if legend_fontsize is not None and (
-        not isinstance(legend_fontsize, (int, float, str))
+        not isinstance(legend_fontsize, int | float | str)
         or (isinstance(legend_fontsize, str) and legend_fontsize not in font_sizes)
     ):
         readable_font_sizes = ", ".join(font_sizes[:-1]) + ", or " + font_sizes[-1]
@@ -1476,22 +1474,22 @@ def _validate_show_parameters(
     if fig is not None and not isinstance(fig, Figure):
         raise TypeError("Parameter 'fig' must be a matplotlib.figure.Figure.")
 
-    if title is not None and not isinstance(title, (list, str)):
+    if title is not None and not isinstance(title, list | str):
         raise TypeError("Parameter 'title' must be a string or a list of strings.")
 
     if not isinstance(share_extent, bool):
         raise TypeError("Parameter 'share_extent' must be a boolean.")
 
-    if not isinstance(pad_extent, (int, float)):
+    if not isinstance(pad_extent, int | float):
         raise TypeError("Parameter 'pad_extent' must be numeric.")
 
-    if ax is not None and not isinstance(ax, (Axes, list)):
+    if ax is not None and not isinstance(ax, Axes | list):
         raise TypeError("Parameter 'ax' must be a matplotlib.axes.Axes or a list of Axes.")
 
     if not isinstance(return_ax, bool):
         raise TypeError("Parameter 'return_ax' must be a boolean.")
 
-    if save is not None and not isinstance(save, (str, Path)):
+    if save is not None and not isinstance(save, str | Path):
         raise TypeError("Parameter 'save' must be a string or a pathlib.Path.")
 
 
@@ -1501,7 +1499,6 @@ def _type_check_params(param_dict: dict[str, Any], element_type: str) -> dict[st
             "Parameter 'element' must be a string. If you want to display more elements, pass `element` "
             "as `None` or chain pl.render(...).pl.render(...).pl.show()"
         )
-
     if element_type == "images":
         param_dict["element"] = [element] if element is not None else list(param_dict["sdata"].images.keys())
     elif element_type == "labels":
@@ -1511,10 +1508,10 @@ def _type_check_params(param_dict: dict[str, Any], element_type: str) -> dict[st
     elif element_type == "shapes":
         param_dict["element"] = [element] if element is not None else list(param_dict["sdata"].shapes.keys())
 
-    if (channel := param_dict.get("channel")) is not None and not isinstance(channel, (list, str, int)):
+    if (channel := param_dict.get("channel")) is not None and not isinstance(channel, list | str | int):
         raise TypeError("Parameter 'channel' must be a string, an integer, or a list of strings or integers.")
     if isinstance(channel, list):
-        if not all(isinstance(c, (str, int)) for c in channel):
+        if not all(isinstance(c, str | int) for c in channel):
             raise TypeError("Each item in 'channel' list must be a string or an integer.")
         if not all(isinstance(c, type(channel[0])) for c in channel):
             raise TypeError("Each item in 'channel' list must be of the same type, either string or integer.")
@@ -1539,13 +1536,13 @@ def _type_check_params(param_dict: dict[str, Any], element_type: str) -> dict[st
         param_dict["col_for_color"] = None
 
     if outline_width := param_dict.get("outline_width"):
-        if not isinstance(outline_width, (float, int)):
+        if not isinstance(outline_width, float | int):
             raise TypeError("Parameter 'outline_width' must be numeric.")
         if outline_width < 0:
             raise ValueError("Parameter 'outline_width' cannot be negative.")
 
     if (outline_alpha := param_dict.get("outline_alpha")) and (
-        not isinstance(outline_alpha, (float, int)) or not 0 <= outline_alpha <= 1
+        not isinstance(outline_alpha, float | int) or not 0 <= outline_alpha <= 1
     ):
         raise TypeError("Parameter 'outline_alpha' must be numeric and between 0 and 1.")
 
@@ -1553,13 +1550,13 @@ def _type_check_params(param_dict: dict[str, Any], element_type: str) -> dict[st
         raise ValueError("Parameter 'contour_px' must be a positive number.")
 
     if (alpha := param_dict.get("alpha")) is not None:
-        if not isinstance(alpha, (float, int)):
+        if not isinstance(alpha, float | int):
             raise TypeError("Parameter 'alpha' must be numeric.")
         if not 0 <= alpha <= 1:
             raise ValueError("Parameter 'alpha' must be between 0 and 1.")
 
     if (fill_alpha := param_dict.get("fill_alpha")) is not None:
-        if not isinstance(fill_alpha, (float, int)):
+        if not isinstance(fill_alpha, float | int):
             raise TypeError("Parameter 'fill_alpha' must be numeric.")
         if fill_alpha < 0:
             raise ValueError("Parameter 'fill_alpha' cannot be negative.")
@@ -1569,7 +1566,7 @@ def _type_check_params(param_dict: dict[str, Any], element_type: str) -> dict[st
     param_dict["cmap"] = cmap
 
     if (groups := param_dict.get("groups")) is not None:
-        if not isinstance(groups, (list, str)):
+        if not isinstance(groups, list | str):
             raise TypeError("Parameter 'groups' must be a string or a list of strings.")
         if isinstance(groups, str):
             param_dict["groups"] = [groups]
@@ -1581,7 +1578,7 @@ def _type_check_params(param_dict: dict[str, Any], element_type: str) -> dict[st
     if isinstance((palette := param_dict["palette"]), list):
         if not all(isinstance(p, str) for p in palette):
             raise ValueError("If specified, parameter 'palette' must contain only strings.")
-    elif isinstance(palette, (str, type(None))) and "palette" in param_dict:
+    elif isinstance(palette, str | type(None)) and "palette" in param_dict:
         param_dict["palette"] = [palette] if palette is not None else None
 
     if element_type in ["shapes", "points", "labels"] and (palette := param_dict.get("palette")) is not None:
@@ -1595,9 +1592,9 @@ def _type_check_params(param_dict: dict[str, Any], element_type: str) -> dict[st
             )
 
     if isinstance(cmap, list):
-        if not all(isinstance(c, (Colormap, str)) for c in cmap):
+        if not all(isinstance(c, Colormap | str) for c in cmap):
             raise TypeError("Each item in 'cmap' list must be a string or a Colormap.")
-    elif isinstance(cmap, (Colormap, str, type(None))):
+    elif isinstance(cmap, Colormap | str | type(None)):
         if "cmap" in param_dict:
             param_dict["cmap"] = [cmap] if cmap is not None else None
     else:
@@ -1611,20 +1608,20 @@ def _type_check_params(param_dict: dict[str, Any], element_type: str) -> dict[st
     if (norm := param_dict.get("norm")) is not None:
         if element_type in ["images", "labels"] and not isinstance(norm, Normalize):
             raise TypeError("Parameter 'norm' must be of type Normalize.")
-        if element_type in ["shapes", "points"] and not isinstance(norm, (bool, Normalize)):
+        if element_type in ["shapes", "points"] and not isinstance(norm, bool | Normalize):
             raise TypeError("Parameter 'norm' must be a boolean or a mpl.Normalize.")
 
     if (scale := param_dict.get("scale")) is not None:
         if element_type in ["images", "labels"] and not isinstance(scale, str):
             raise TypeError("Parameter 'scale' must be a string if specified.")
         if element_type == "shapes":
-            if not isinstance(scale, (float, int)):
+            if not isinstance(scale, float | int):
                 raise TypeError("Parameter 'scale' must be numeric.")
             if scale < 0:
                 raise ValueError("Parameter 'scale' must be a positive number.")
 
     if size := param_dict.get("size"):
-        if not isinstance(size, (float, int)):
+        if not isinstance(size, float | int):
             raise TypeError("Parameter 'size' must be numeric.")
         if size < 0:
             raise ValueError("Parameter 'size' must be a positive number.")
@@ -1632,9 +1629,28 @@ def _type_check_params(param_dict: dict[str, Any], element_type: str) -> dict[st
     if param_dict.get("table_name") and not isinstance(param_dict["table_name"], str):
         raise TypeError("Parameter 'table_name' must be a string .")
 
-    if param_dict.get("method") not in ["matplotlib", "datashader", None]:
+    # like this because the following would assign True/False to 'method'
+    if (method := param_dict.get("method")) not in ["matplotlib", "datashader", None]:
         raise ValueError("If specified, parameter 'method' must be either 'matplotlib' or 'datashader'.")
 
+    valid_ds_reduction_methods = [
+        "sum",
+        "mean",
+        "any",
+        "count",
+        # "m2", -> not intended to be used alone (see https://datashader.org/api.html#datashader.reductions.m2)
+        # "mode", -> not supported for points (see https://datashader.org/api.html#datashader.reductions.mode)
+        "std",
+        "var",
+        "max",
+        "min",
+    ]
+    if (ds_reduction := param_dict.get("ds_reduction")) and (ds_reduction not in valid_ds_reduction_methods):
+        raise ValueError(f"Parameter 'ds_reduction' must be one of the following: {valid_ds_reduction_methods}.")
+
+    if method == "datashader" and ds_reduction is None:
+        param_dict["ds_reduction"] = "sum"
+
     return param_dict
 
 
@@ -1711,6 +1727,7 @@ def _validate_points_render_params(
     norm: Normalize | None,
     size: float | int,
     table_name: str | None,
+    ds_reduction: str | None,
 ) -> dict[str, dict[str, Any]]:
     param_dict: dict[str, Any] = {
         "sdata": sdata,
@@ -1724,6 +1741,7 @@ def _validate_points_render_params(
         "norm": norm,
         "size": size,
         "table_name": table_name,
+        "ds_reduction": ds_reduction,
     }
     param_dict = _type_check_params(param_dict, "points")
 
@@ -1752,6 +1770,7 @@ def _validate_points_render_params(
 
         element_params[el]["palette"] = param_dict["palette"] if param_dict["col_for_color"] is not None else None
         element_params[el]["groups"] = param_dict["groups"] if param_dict["col_for_color"] is not None else None
+        element_params[el]["ds_reduction"] = param_dict["ds_reduction"]
 
     return element_params
 
@@ -1772,6 +1791,7 @@ def _validate_shape_render_params(
     scale: float | int,
     table_name: str | None,
     method: str | None,
+    ds_reduction: str | None,
 ) -> dict[str, dict[str, Any]]:
     param_dict: dict[str, Any] = {
         "sdata": sdata,
@@ -1789,6 +1809,7 @@ def _validate_shape_render_params(
         "scale": scale,
         "table_name": table_name,
         "method": method,
+        "ds_reduction": ds_reduction,
     }
     param_dict = _type_check_params(param_dict, "shapes")
 
@@ -1822,6 +1843,7 @@ def _validate_shape_render_params(
         element_params[el]["palette"] = param_dict["palette"] if param_dict["col_for_color"] is not None else None
         element_params[el]["groups"] = param_dict["groups"] if param_dict["col_for_color"] is not None else None
         element_params[el]["method"] = param_dict["method"]
+        element_params[el]["ds_reduction"] = param_dict["ds_reduction"]
 
     return element_params
 
@@ -1897,18 +1919,18 @@ def _validate_image_render_params(
 
         if isinstance(palette := param_dict["palette"], list):
             if len(palette) == 1:
-                palette_length = len(channel) if channel is not None else len(spatial_element.c)
+                palette_length = len(channel) if channel is not None else len(spatial_element_ch)
                 palette = palette * palette_length
-            if (channel is not None and len(palette) != len(channel)) and len(palette) != len(spatial_element.c):
+            if (channel is not None and len(palette) != len(channel)) and len(palette) != len(spatial_element_ch):
                 palette = None
         element_params[el]["palette"] = palette
         element_params[el]["na_color"] = param_dict["na_color"]
 
         if (cmap := param_dict["cmap"]) is not None:
             if len(cmap) == 1:
-                cmap_length = len(channel) if channel is not None else len(spatial_element.c)
+                cmap_length = len(channel) if channel is not None else len(spatial_element_ch)
                 cmap = cmap * cmap_length
-            if (channel is not None and len(cmap) != len(channel)) or len(cmap) != len(spatial_element.c):
+            if (channel is not None and len(cmap) != len(channel)) or len(cmap) != len(spatial_element_ch):
                 cmap = None
         element_params[el]["cmap"] = cmap
         element_params[el]["norm"] = param_dict["norm"]
@@ -1949,13 +1971,13 @@ def _is_coercable_to_float(series: pd.Series) -> bool:
 
 
 def _ax_show_and_transform(
-    array: MaskedArray[np.float64, Any],
+    array: MaskedArray[tuple[int, ...], Any],
     trans_data: CompositeGenericTransform,
     ax: Axes,
     alpha: float | None = None,
     cmap: ListedColormap | LinearSegmentedColormap | None = None,
     zorder: int = 0,
-) -> None:
+) -> matplotlib.image.AxesImage:
     if not cmap and alpha is not None:
         im = ax.imshow(
             array,
@@ -1970,6 +1992,7 @@ def _ax_show_and_transform(
             zorder=zorder,
         )
         im.set_transform(trans_data)
+    return im
 
 
 def set_zero_in_cmap_to_transparent(cmap: Colormap | str, steps: int | None = None) -> ListedColormap:
@@ -1994,6 +2017,152 @@ def set_zero_in_cmap_to_transparent(cmap: Colormap | str, steps: int | None = No
     return ListedColormap(colors)
 
 
+def _get_extent_and_range_for_datashader_canvas(
+    spatial_element: SpatialElement, coordinate_system: str, ax: Axes, fig_params: FigParams
+) -> tuple[Any, Any, list[Any], list[Any], Any]:
+    extent = get_extent(spatial_element, coordinate_system=coordinate_system)
+    x_ext = [min(0, extent["x"][0]), extent["x"][1]]
+    y_ext = [min(0, extent["y"][0]), extent["y"][1]]
+    previous_xlim = ax.get_xlim()
+    previous_ylim = ax.get_ylim()
+    # increase range if sth larger was rendered on the axis before
+    if _mpl_ax_contains_elements(ax):
+        x_ext = [min(x_ext[0], previous_xlim[0]), max(x_ext[1], previous_xlim[1])]
+        y_ext = (
+            [
+                min(y_ext[0], previous_ylim[1]),
+                max(y_ext[1], previous_ylim[0]),
+            ]
+            if ax.yaxis_inverted()
+            else [
+                min(y_ext[0], previous_ylim[0]),
+                max(y_ext[1], previous_ylim[1]),
+            ]
+        )
+
+    # compute canvas size in pixels close to the actual image size to speed up computation
+    plot_width = x_ext[1] - x_ext[0]
+    plot_height = y_ext[1] - y_ext[0]
+    plot_width_px = int(round(fig_params.fig.get_size_inches()[0] * fig_params.fig.dpi))
+    plot_height_px = int(round(fig_params.fig.get_size_inches()[1] * fig_params.fig.dpi))
+    factor: float
+    factor = np.min([plot_width / plot_width_px, plot_height / plot_height_px])
+    plot_width = int(np.round(plot_width / factor))
+    plot_height = int(np.round(plot_height / factor))
+
+    return plot_width, plot_height, x_ext, y_ext, factor
+
+
+def _create_image_from_datashader_result(
+    ds_result: ds.transfer_functions.Image, factor: float, ax: Axes
+) -> tuple[MaskedArray[tuple[int, ...], Any], matplotlib.transforms.CompositeGenericTransform]:
+    # create SpatialImage from datashader output to get it back to original size
+    rgba_image_data = ds_result.to_numpy().base
+    rgba_image_data = np.transpose(rgba_image_data, (2, 0, 1))
+    rgba_image = Image2DModel.parse(
+        rgba_image_data,
+        dims=("c", "y", "x"),
+        transformations={"global": Scale([1, factor, factor], ("c", "y", "x"))},
+    )
+
+    _, trans_data = _prepare_transformation(rgba_image, "global", ax)
+
+    rgba_image = np.transpose(rgba_image.data.compute(), (1, 2, 0))  # type: ignore[attr-defined]
+    rgba_image = ma.masked_array(rgba_image)  # type conversion for mypy
+
+    return rgba_image, trans_data
+
+
+def _datashader_aggregate_with_function(
+    reduction: Literal["sum", "mean", "any", "count", "std", "var", "max", "min"] | None,
+    cvs: Canvas,
+    spatial_element: GeoDataFrame | dask.dataframe.core.DataFrame,
+    col_for_color: str | None,
+    element_type: Literal["points", "shapes"],
+) -> DataArray:
+    """
+    When shapes or points are colored by a continuous value during rendering with datashader.
+
+    This function performs the aggregation using the user-specified reduction method.
+
+    Parameters
+    ----------
+    reduction: String specifying the datashader reduction method to be used.
+        If None, "sum" is used as default.
+    cvs: Canvas object previously created with ds.Canvas()
+    spatial_element: geo or dask dataframe with the shapes or points to render
+    col_for_color: name of the column containing the values by which to color
+    element_type: tells us if this function is called from _render_shapes() or _render_points()
+    """
+    if reduction is None:
+        reduction = "sum"
+
+    reduction_function_map = {
+        "sum": ds.sum,
+        "mean": ds.mean,
+        "any": ds.any,
+        "count": ds.count,
+        "std": ds.std,
+        "var": ds.var,
+        "max": ds.max,
+        "min": ds.min,
+    }
+
+    try:
+        reduction_function = reduction_function_map[reduction](column=col_for_color)
+    except KeyError as e:
+        raise ValueError(
+            f"Reduction '{reduction}' is not supported. Please use one of: {', '.join(reduction_function_map.keys())}."
+        ) from e
+
+    element_function_map = {
+        "points": cvs.points,
+        "shapes": cvs.polygons,
+    }
+
+    try:
+        element_function = element_function_map[element_type]
+    except KeyError as e:
+        raise ValueError(f"Element type '{element_type}' is not supported. Use 'points' or 'shapes'.") from e
+
+    if element_type == "points":
+        points_aggregate = element_function(spatial_element, "x", "y", agg=reduction_function)
+        if reduction == "any":
+            # replace False/True by nan/1
+            points_aggregate = points_aggregate.astype(int)
+            points_aggregate = points_aggregate.where(points_aggregate > 0)
+        return points_aggregate
+
+    # is shapes
+    return element_function(spatial_element, geometry="geometry", agg=reduction_function)
+
+
+def _datshader_get_how_kw_for_spread(
+    reduction: Literal["sum", "mean", "any", "count", "std", "var", "max", "min"] | None
+) -> str:
+    # Get the best input for the how argument of ds.tf.spread(), needed for numerical values
+    reduction = reduction or "sum"
+
+    reduction_to_how_map = {
+        "sum": "add",
+        "mean": "source",
+        "any": "source",
+        "count": "add",
+        "std": "source",
+        "var": "source",
+        "max": "max",
+        "min": "min",
+    }
+
+    if reduction not in reduction_to_how_map:
+        raise ValueError(
+            f"Reduction {reduction} is not supported, please use one of the following: sum, mean, any, count"
+            ", std, var, max, min."
+        )
+
+    return reduction_to_how_map[reduction]
+
+
 def _robust_get_value(
     sdata: sd.SpatialData,
     origin: _ValueOrigin,
@@ -2007,3 +2176,14 @@ def _robust_get_value(
         return get_values_point_table(sdata=sdata, origin=origin, table_name=table_name)
     vals = get_values(value_key=value_to_plot, sdata=sdata, element_name=element_name, table_name=table_name)
     return vals[value_to_plot]
+
+
+def _prepare_transformation(
+    element: DataArray | GeoDataFrame | dask.dataframe.core.DataFrame, coordinate_system: str, ax: Axes | None = None
+) -> tuple[matplotlib.transforms.Affine2D, matplotlib.transforms.CompositeGenericTransform | None]:
+    trans = get_transformation(element, get_all=True)[coordinate_system]
+    affine_trans = trans.to_affine_matrix(input_axes=("x", "y"), output_axes=("x", "y"))
+    trans = mtransforms.Affine2D(matrix=affine_trans)
+    trans_data = trans + ax.transData if ax is not None else None
+
+    return trans, trans_data
